/*
 *  $Id: pkt_int.c,v 1.1.1.1 2000/03/18 15:33:42 gvanem Exp $
 *
 *  libnet
 *  pkt_int.c
 *
 *  Copyright (c) 2000 Gisle Vanem <giva@bgnett.no>
 *
 *  All rights reserved.
 *
 * Copyright (c) 1990, 1991, 1992, 1993, 1994, 1995, 1996
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that: (1) source code distributions
 * retain the above copyright notice and this paragraph in its entirety, (2)
 * distributions including binary code include the above copyright notice and
 * this paragraph in its entirety in the documentation or other materials
 * provided with the distribution, and (3) all advertising materials mentioning
 * features or use of this software display the following acknowledgement:
 * ``This product includes software developed by the University of California,
 * Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
 * the University nor the names of its contributors may be used to endorse
 * or promote products derived from this software without specific prior
 * written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Libnet interface for real-mode PKTDRVR (packet driver)
 * This module performs link-layer transmissions using functions in
 * libpcap.a (module pktdrvr.c)
 */

#if !defined(__DJGPP__)
#error This is for djgpp and DOS only
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <pc-types.h>  /* in ./pcap directory */
#include <pc-int.h>    /* ditto */
#include <pktdrvr.h>   /* ditto */

#include "../include/libnet.h"

static char dev[] = "/dev/pkt";

static void pkt_close (void)
{
  PktExitDriver();
  pkt.handle = 0;
}

struct libnet_link_int *libnet_open_link_interface (char *device, char *ebuf)
{
  struct libnet_link_int *l;
  PKT_RX_MODE mode = PDRX_DIRECT; /* !!was PDRX_ALL_PACKETS */

  l = calloc (sizeof(*l), 1);
  if (!l)
  {
    strcpy (ebuf, ll_strerror(errno));
    return (NULL);
  }

  if (!PktInitDriver(mode, 0))
  {
    free (l);
    sprintf (ebuf, "%s: %s", dev, ll_strerror(errno));
    return (NULL);
  }
  l->fd       = pkt.handle;
  l->linktype = DLT_EN10MB;

  atexit (pkt_close);

  PktResetStatistics (l->fd);
  PktQueueBusy (FALSE);
  return (l);
}

struct ether_addr *
libnet_get_hwaddr (struct libnet_link_int *l, const u_char *device, char *ebuf)
{
  struct ether_addr *ea;

  if (!l || l->fd != pkt.handle)
  {
    if (ebuf)
       sprintf (ebuf, "get_hwaddr: %s not open", dev);
    return (NULL);
  }

  ea = malloc (sizeof(*ea));
  if (!ea)
  {
    if (ebuf)
       strcpy (ebuf, "get_hwaddr: allocation failed");
    return (NULL);
  }

  if (!PktGetAddress((ETHER*)&ea->ether_addr_octet))
  {
    if (ebuf)
       strcpy (ebuf, "get_hwaddr: PktGetAddress() failed");
    free (ea);
    return (NULL);
  }

  (void)device;  /* argsused */
  return (ea);
}

int libnet_close_link_interface (struct libnet_link_int *l)
{
  pkt_close();
  l->fd = 0;
  return (1);
}

int libnet_write_link_layer (
           struct libnet_link_int *l, /* libnet ctrl struct */
           const u_char *device,      /* device name ("pkt") */
           u_char       *buf,         /* transmit buffer */
           int           len)         /* buffer length */
{
  if (!PktTransmit(buf, len))
     return (0);
  return (len);
}

int
libnet_select_device(struct sockaddr_in *sin, u_char **device, u_char *errbuf)
{
  *device = "pkt";
  (void)sin;
  (void)errbuf;
  return (1);
}

